// CrackUp.cpp : Defines the class behaviors for the application.
//
// CrackUp version 1.0
// Copyright  1998 Ziff-Davis, Inc.
// First published in PC Magazine, US Edition, November 3, 1998
// Written by Bob Flanders and Gregory A. Wolking

#include "stdafx.h"
#include "CrackUp.h"
#include "CUDlg.h"
#include "ScanDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//  Static variables (used only in this module)
static PBPB				s_pB;				// pointer to BPB
static BOOL				s_bSystemFile;		// TRUE if system file
static CScanMsg*		s_lpScanMsg;		// pointer to scan message window
static UCHAR*			s_lpacFAT;			// fat area
static UCHAR			s_pszDirName[12],	// directory name
						s_pszStatus[60];	// status message
static NOTIFYICONDATA	s_ni;				// structure used by ShellNotifyIcon()

// *****************************************************************************
// Global variables (shared among two or more modules)
// *****************************************************************************
HANDLE				g_hVwin32;				// handle for VWIN32
HWND				g_hMasterWindow;		// main window handle
CRITICAL_SECTION	g_csBG,					// background is running
					g_csScanning;			// a scan is scan in progress
BOOL			g_bPopup = TRUE,			// popups wanted
				g_bQuickMode = FALSE,		// flag for "/QUICK" cmdline switch
				g_bFragBelowMax = TRUE,		// true if worst below maximum
				g_bScanInProgress = FALSE,	// scan is happening
				g_bBackground = TRUE,		// use background thread
				g_bRunThread = TRUE,		// run thread until FALSE
				g_bStopScanNow = FALSE,		// quit the scan
				g_bMessageActive = FALSE,	// signals a message box is active.
				g_bNewWindows = FALSE,		// true if OSR2
				g_bAtLeastOne = FALSE;		// true after first drive is scanned.
WORD			g_wMaxFrag = 30,			// maximum frag allowed
				g_wNDrivesSelected = 0,		// number of drives selected
				g_wScanInterval = DFLTIME,	// hours between scans
				g_wTimeTicks = 360,			// "ticks" left this hour.
				g_wHoursLeft = DFLTIME;		// time left until resume
int				g_intFormula = 0;			// Formula for frag warning.
HANDLE			g_hMyThread;				// thread handle
OSVERSIONINFO	g_stOsVer;					// version of the os
DRVTABENT		g_stDrvTbl[MAXDRIVES];		// drive info array
PDRVTABENT		g_pZ;						// drive info pointer
WORD			g_wNDrives = 0;				// number of valid drives found
float			g_fWorstFrag;				// worst fragmentation ever
CString			g_strIniFile;				// ini file name
HICON			g_hIcons[4] = 				// icons for tray icon animation.
				{
					NULL, NULL, NULL, NULL
				};
CDialog*		g_pRptWin = NULL;			// pointer to property sheet
CCrackUpDlg		g_dlgMain;					// main dialog object
CString			g_strMaxMsg;		// max exceeded message

///////////////////////////////
// Global Function Declarations
///////////////////////////////
void AnalyzeChain(DWORD dwCluster, BOOL bSystemFile);
void BGScan();
void ChangeTip(HWND hWnd,/* UINT uID,*/ LPSTR lpszTip);
void CheckFrag(PDRVTABENT pstDrv);
void CheckLevels();
void CountFreeSpaces(void);
void DrvAnalyze(PDRVTABENT pstDrv, DWORD dwDirCluster);
void ErrorMsg(char * szMsg);
void RotateIcon(int iIndex = -1);
void ScanDrives(CScanMsg* lpSx);
void StartBgScan();
void WriteProfile(void);
BOOL BuildSystray(HWND hwnd, UINT uID, HICON hicon, LPSTR lpszTip);
BOOL DrvEligible(char* szDrv, PDRVTABENT pstDrvTab);
BOOL KillSystray(HWND hwnd, UINT uID);
BOOL NewReadSectors(HANDLE hDev, char cDrive, DWORD dwStartSector, WORD wSectors, LPBYTE lpSectBuff);
BOOL OldReadSectors(HANDLE hDev, char cDrive, DWORD dwStartSector, WORD wSectors, LPBYTE lpSectBuff);
BOOL ReadSectors(HANDLE hDev, char cDrive, DWORD dwStartSector, WORD wSectors, LPBYTE lpSectBuff);
WORD DetermineFatType(HANDLE hDev, char cDrive, PDRVTABENT pstDrvTab);
DWORD NextCluster(DWORD dwCluster);
DWORD NextClusVal(DWORD dwCluster);
float DefragsPct(PDRVTABENT pD, int iFormula);

////////////////////////
// Global function definitions
////////////////////////

// *****************************************************************************
// Display an error message
// Note: Used only during initialization
// *****************************************************************************

void	ErrorMsg(char *szMsg)						// message to display
{													
	MessageBox(NULL, szMsg, ERMTITLE,					// show the message box
			MB_OK | MB_ICONEXCLAMATION | MB_TOPMOST);
}

// *****************************************************************************
// Scan drives
// *****************************************************************************
void	ScanDrives(CScanMsg *lpSx)			// scan the drives
{
	WORD		wI;									// work word
	PDRVTABENT	pX;									// drv table pointers

	EnterCriticalSection(&g_csScanning);			// show scan is running
	g_bStopScanNow = FALSE;							// don't honor false stop
	g_bScanInProgress = TRUE;						// show scan started
	s_lpScanMsg = lpSx;									// save window pointer
	for (wI = 0; (wI < g_wNDrives) && !g_bStopScanNow;)	// for each drive found ..
	{												
		pX = &g_stDrvTbl[wI++];						// save drive table entry

		if (pX->bCheckDrive)						// q. check this drive?
		{											// a. yes ..
			CheckFrag(pX);							// check the fragmentation
			if (g_bStopScanNow)						// q. stop requested?
				break;								// a. yes .. stop right now
			if (pX->dwClusInFrags)					// q. fragmented clusters?
				pX->fFragLvl=(float) ((DWORD) (DefragsPct(pX, g_intFormula)// a. yes .. calculate
							* 100.0f));				// .. fragmentation as a percentage
			else									// else ..
				pX->fFragLvl = 0.0;					// .. show no fragmentation

		}
	}
	if (!g_bQuickMode)			// If not in quick mode,
		RotateIcon(0);			// restore master tray icon.
	if (!g_bStopScanNow)		// If scan was not aborted,
		::PostMessage(g_hMasterWindow, MYWM_CHECKLEVELS,	// post a message
							  IDM_CHKFRAG, 0);			// .. telling main to check for fragmentation
	g_wTimeTicks = 360;								// reset ticks in this hour
	g_wHoursLeft = g_wScanInterval;					// .. and hours to next scan
	g_bScanInProgress = FALSE;						// scan not happening
	g_bStopScanNow = FALSE;							// ensure next scan continues unless stopped
	LeaveCriticalSection(&g_csScanning);			// .. scanning complete
}

// *****************************************************************************
// CheckLevels()
//
//   Determine if we must display a warning message & do so if necessary
//
// *****************************************************************************
void	CheckLevels()
{
	float		fNewWorst = 0.0;					// new worst level
	WORD		wI,									// work integer
				wNDrivesChecked = 0;				// number of drives checked
	PDRVTABENT	pW, pX;								// worst drive, work pointers
	char		acDrv[30],							// work area for popup message
				acX[10];							// work for frag level

	for (wI = 0; (wI < g_wNDrives);)					// for each drive found ..
	{												
		pX = &g_stDrvTbl[wI++];						// save drive table entry

		if (pX->bCheckDrive)					// q. check this drive?
		{										// a. yes ..
			if (pX->fFragLvl > fNewWorst)		// q. new worst level this scan?
			{									// a. yes..
				pW = pX;						// .. save the winning drive
				fNewWorst = pX->fFragLvl;		// .. replace old NewWorst
			}
			wNDrivesChecked++;			// show it checked
		}
	}
	if (wNDrivesChecked == 0)						// q. any drives checked?
	{
		strcpy((char *) s_pszStatus,				// a. no .. inform the user
						"No drives scanned");	// .. there were none
	}
	else 											// else ...
	{
		if (fNewWorst == 0.0)		// No drives fragmented?
			strcpy((char *) s_pszStatus, "No drives fragmented");
		else
			sprintf((char *) s_pszStatus,				// .. build percentage message
				"Worst drive %c:%.1f%% fragmented",		// .. use this format
				pW->cDrv,								// .. drive letter
				pW->fFragLvl);							// .. and frag level
	}
	if (!g_bQuickMode)								// If not in quick mode,
		ChangeTip(g_hMasterWindow,						// change the tip
				(char *) s_pszStatus);				// .. use this message
	if (wNDrivesChecked == 0)						// q. drives checked?
		return;										// a. no .. leave now
	g_bFragBelowMax = g_bFragBelowMax 				// set inidicator ..
					|| (fNewWorst < g_fWorstFrag); 	// .. if current worst
						       						// .. drops below worst ever seen
	if (!g_bPopup || !g_bFragBelowMax)				// q. popup turned on & gun cocked?
		return;										// a. no .. leave now.

	if (fNewWorst < (g_fWorstFrag + 1.0))			// q. is it over the worst by 1 percent?
		return;										// a. no .. no need for a popup now

	g_fWorstFrag = fNewWorst;							// reset worst ever seen
	g_strMaxMsg.Empty();								// empty the popup message

	for (wI = 0; (wI < g_wNDrives);)					// for each drive found ..
	{												
		pX = &g_stDrvTbl[wI++];						// save drive table entry

		if (pX->bCheckDrive)						// q. check this drive?
		{										// a. yes ..
			if ((pX->fFragLvl-.10)>=(float)g_wMaxFrag)// .. and max frag level exceeded by .1%?
			{									// a. yes ..
				sprintf(acDrv,	"            	"	// start with centering spaces
						"%c: (%4.1f%%)\n",			// .. build message
							pX->cDrv,				// .. showing drive
							pX->fFragLvl);			// .. and frag level

				if (g_strMaxMsg.GetLength() == 0)		// q. any length yet?
				{								// a. no .. 
					ltoa(g_wMaxFrag, acX, 10);		// .. convert the max value
					g_strMaxMsg =						// .. build the msgbox
						"Fragmentation "			// .. start message
						"exceeds ";					// .. with exceeds
					g_strMaxMsg += acX;				// .. the max
					g_strMaxMsg += "%:\n";			// .. error
				}
				g_strMaxMsg += acDrv;					// .. add the bad drive
			}
		}
	}

	if (g_strMaxMsg.GetLength()) 						// q. any msg to display?
	{													// a. yes ..
		if (g_pRptWin == NULL)				// Property Sheet not open?
		{
			g_bMessageActive = TRUE;		// Set flag for tray icon handler.
			::MessageBox(					// Use a plain old message box.
			// Set proper owner for message box. If property sheed is active,
			// we want the message to be modal relative to the sheet, not
			// our background window.
			g_hMasterWindow,
			g_strMaxMsg,					// .. with a message
			"CrackUp Notice",				// .. this title
			MB_OK | MB_ICONEXCLAMATION		// .. and an exclamation
				 | MB_TOPMOST);				// .. put it on top.
			g_bMessageActive = FALSE;
		}
		else		// If property sheet is open, have it display the message.
			g_pRptWin->PostMessage(MYWM_SHOWWARNING, 0, 0);
		g_bFragBelowMax = FALSE;						// show the gun has been fired
	}
}

// *****************************************************************************
// OldReadSectors(hDev, cDrive, dwStartSector, wSectors, lpSectBuff)
//
//   Read one or more sectors from a logical drive
//   using Int 25h through VWIN32
//
// Parameters:
//    hDev			Handle of VWIN32
//    cDrive		The MS-DOS drive letter. Case doesn't matter
//    dwStartSector	First sector to read
//    wSectors		Number of sectors to read
//    lpSectBuff	Buffer for read data
//
//   Return Value:
//      Returns TRUE if successful, or FALSE if failure.
// *****************************************************************************
BOOL OldReadSectors (HANDLE hDev,				// VWIND32 device
					 char   cDrive,				// drive letter
                     DWORD  dwStartSector,		// start sector
                     WORD   wSectors,			// number of sectors
                     LPBYTE lpSectBuff)			// sector buffer
{
	BOOL           bResult;							// result from VWIN32
	DWORD          dwW;								// work DW
	DIOC_REGISTERS reg = {0};						// registers for int25
	DISKIO         dio = {0};						// disk io block

	dio.dwStartSector = dwStartSector;				// set the starting sector
	dio.wSectors      = wSectors;					// .. and number of sectors
	dio.lpBuffer      = lpSectBuff;					// .. and where the data goes	

	reg.reg_EAX = (cDrive & 0x5f) - 'A';			// Int 25h drive numbers are 0-based.
	reg.reg_EBX = (DWORD) &dio;						// set pointer to disk IO structure
	reg.reg_ECX = 0xFFFF;							// tell int 25h to use DISKIO struct

	bResult = DeviceIoControl(hDev,					// ask VWIN32
						VWIN32_DIOC_DOS_INT25,		// .. to read some sectors 
						&reg, sizeof(reg),			// .. use registers
						&reg, sizeof(reg),			// .. on both input and output
						&dwW, 0);					// .. and returned length

	bResult = bResult &&							// successful
					!(reg.reg_Flags & CARRY_FLAG);	// .. if carry not set
	return bResult;									// .. return the result
}

// *****************************************************************************
// NewReadSectors(hDev, cDrive, dwStartSector, wSectors, lpSectBuff)
//
// Read one or more sectors using Int 21h function 7305h
//
// Parameters:
//	hDev			Handle of VWIN32
// 	cDrive			MS-DOS drive letter. Case doesn't matter
// 	dwStartSector	First sector to read.
// 	wSectors		Number of sectors to read.
// 	lpSectBuff      Buffer to read into
// 
// Return Value:
// 	Returns TRUE if successful, or FALSE if failure.
// *****************************************************************************
BOOL NewReadSectors (HANDLE hDev,				// VWIND32 device
                     char   cDrive,				// drive letter
                     DWORD  dwStartSector,		// start sector
                     WORD   wSectors,			// number of sectors
                     LPBYTE lpSectBuff)			// sector buffer
{
	BOOL           bResult;							// result from IO call
	DWORD          dwW;								// work dw
	DIOC_REGISTERS reg = {0};						// register block
	DISKIO         dio;								// disk IO block

	dio.dwStartSector = dwStartSector;				// set the starting sector
	dio.wSectors      = wSectors;					// .. and number to read
	dio.lpBuffer      = lpSectBuff;					// .. and the input buffer

	reg.reg_EAX = 0x7305;							// Ext_ABSDiskReadWrite OSR2
	reg.reg_EBX = (DWORD) &dio;						// disk IO block address
	reg.reg_ECX = 0xFFFFFFFF;						// Using a disk IO block
	reg.reg_EDX = ((cDrive & 0x5f) - 'A') + 1;		

	bResult = DeviceIoControl(hDev,					// call VWIN32
					VWIN32_DIOC_DOS_DRIVEINFO,		// .. issue Int 21h, fn 7305h 
					&reg, sizeof(reg),				// .. send our registers
					&reg, sizeof(reg),				// .. and get em back too
					&dwW, 0);						// .. and returned length
	bResult = bResult &&							// successful
					!(reg.reg_Flags & CARRY_FLAG);	// .. if carry not set
	return bResult;									// return the result
}


// *****************************************************************************
// ReadSectors(hDev, cDrive, dwStartSector, wSectors, lpSectBuff)
//
// Read one or more sectors using appropriate function
//
// Parameters:
//	hDev			Handle of VWIN32
// 	cDrive			MS-DOS drive letter. Case doesn't matter
// 	dwStartSector	First sector to read.
// 	wSectors		Number of sectors to read.
// 	lpSectBuff      Buffer to read into
// 
// Return Value:
// 	Returns TRUE if successful, or FALSE if failure.
// *****************************************************************************
BOOL ReadSectors (HANDLE hDev,					// VWIND32 device
                  char   cDrive,				// drive letter
                  DWORD  dwStartSector,			// start sector
                  WORD   wSectors,				// number of sectors
                  LPBYTE lpSectBuff)			// sector buffer
{
	return g_bNewWindows ?							// q. new version of windows?
				NewReadSectors(hDev,				// a. yes .. use new call	
							   cDrive,				// .. pass the drive
							   dwStartSector,		// .. start sector
							   wSectors,			// .. number of sectors
							   lpSectBuff) 			// .. and sector buffer

				:									// otherwise ..

				OldReadSectors(hDev,				// use old call	
							   cDrive,				// .. pass the drive
							   dwStartSector,		// .. start sector
							   wSectors,			// .. number of sectors
							   lpSectBuff);			// .. and sector buffer
}


// *****************************************************************************
// Write profile strings
// *****************************************************************************
void	WriteProfile(void)							// write out the profile
{
	char		acVal[12];							// work area
	char		buff[10];
	WORD		wN;									// work number
	PDRVTABENT	pD;									// drive table entry

	acVal[0] = '\0';								// init the string
	WritePrivateProfileString("INIT",				// write out [INIT] ..
							  "CONFIGURED",			// .. CONFIGURED= flag
							  "Y",					// .. show configured
							  g_strIniFile);		// .. to our ini file
	WritePrivateProfileString("INIT",				// write out [INIT] ..
							  "MAXFRAGPERCENT",		// .. MAXFRAGPERCENT=
							  itoa(g_wMaxFrag,		// .. value set in config
								   acVal,			// .. using this buffer
								   10),				// .. in base 10	
							  g_strIniFile);		// .. to our ini file
	WritePrivateProfileString("INIT",				// write out [INIT] ..
							  "POPUP",				// .. POPUP=
							  g_bPopup ? "Y" : "N",	// .. yes or no
							  g_strIniFile);		// .. to our ini file
	WritePrivateProfileString("INIT",				// write out [INIT] ..
							  "BACKGROUND",			// .. BACKGROUND=
							  g_bBackground  ?		// .. based on the switch
									   "Y" : "N",	// .. yes or no
							  g_strIniFile);				// .. to our ini file
	WritePrivateProfileString("INIT",				// write out [INIT] ..
							  "BGTIMER",			// .. BGTIMER=
							  itoa(g_wScanInterval,	// .. timer set in config
								   acVal,			// .. using this buffer
								   10),				// .. in base 10	
							  g_strIniFile);		// .. to our ini file
	sprintf(buff, "%u", g_intFormula);
	WritePrivateProfileString("INIT",
							"FORMULA",
							buff, g_strIniFile);
	pD = &g_stDrvTbl[0];							// init drive pointer
	for (wN = g_wNDrives; wN--; pD++)				// for each drive ..
	{
		sprintf(acVal, "DRIVE%c", pD->cDrv);		// build the drive name
		WritePrivateProfileString("INIT",			// write out [INIT] ..
							  acVal,				// .. DRIVEx=
							  pD->bCheckDrive ? 	// .. based on switch
									"Y" : "N",		// .. yes or no
							  g_strIniFile);		// .. to our ini file
		sprintf(acVal, "Formula%c", pD->cDrv);
	}
}



// *****************************************************************************
// BGScan()
//
// Run a scan in the background thread
//
// *****************************************************************************
void BGScan()						// start background scan if enabled
{
	EnterCriticalSection(&g_csBG);	// show BG is active
	while (g_bRunThread)			// while we're supposed to run this
	{												
		SuspendThread(g_hMyThread);	// stop until someone tells us to go
		if (g_bRunThread)			// q. is program ending?
		{
			if (g_pRptWin == NULL)	// Property sheet active?
				ScanDrives(NULL);		// a. no .. Call scan function directly.
			else
				// Otherwise, tell foreground thread to do the scan and go back to sleep.
				::PostMessage(g_hMasterWindow, WM_COMMAND, ID_SCANNOW, 0);
		}
	}
	LeaveCriticalSection(&g_csBG);					// show BG ending
	ExitThread(0);									// .. end the thread gracefully
}


// *****************************************************************************
// StartBg()
//
// Start the background thread.
//
// *****************************************************************************
void StartBgScan()								// start background scan thread
{
	DWORD dwThreadId;
	g_hMyThread = CreateThread(NULL,				// .. '95 doesn't use security
				0,									// .. use same stacksize
				(LPTHREAD_START_ROUTINE) BGScan,	// .. start in BGScan()
				NULL,								// .. pass 1 -- threaded start
				0,									// .. run immediately						
				&dwThreadId);						// .. place thread id here

	SetThreadPriority(g_hMyThread, THREAD_PRIORITY_IDLE);	// set the priority class
}


// *****************************************************************************
// DeterminFatType(hDev, bDrive)
//
// Determine if this is a FAT16 or FAT32 drive
//
// Parameters:
//	hDev			Handle of VWIN32
// 	bDrive			MS-DOS drive letter. Case doesn't matter
// 
// Return Value:
// 	Returns FAT32 if FAT32, FAT16 if FAT16 or UNKNOWN if neither
// *****************************************************************************
WORD DetermineFatType (HANDLE hDev,				// VWIND32 device
                       char   cDrive,			// drive letter
					   PDRVTABENT pstDrvTab)	// drive table entry 
{
	BOOL            bResult;						// result from IO call
	DWORD           dwW;							// work dw
	DWORD			dwClusters,						// number of clusters 
					dwSectors;						// number of sectors
	PBPB		    pstBPB;							// pointer to BPB
	PBOOTREC		pstBoot;						// pointer to boot record

	pstBoot = (PBOOTREC) malloc(DFLSECTORSZ);		// allocate memory for sector
	if (pstBoot == NULL)							// q. allocate ok?
	{												// a. no ..
		ErrorMsg(ERMNOMEM);							// .. tell the public
		ExitProcess(INITERR);						// .. and return to Windows
	}
	bResult = ReadSectors(hDev,						// using VMWIN32 ..
						  cDrive,					// .. read the drive
						  0,						// .. sector 0
						  1, 						// .. for 1 sector
						  (UCHAR *) pstBoot);		// into the boot record
	if (!bResult)									// q. bResult NZ?
	{												// a. no
		dwW = GetLastError();						// get the error number 
		free(pstBoot);								// .. free the work area
		return UNKNOWN;								// .. and mark this drive inelegible	
	}
	pstBPB = &pstBoot->stBPB;						// save BPB address
	pstDrvTab->cDrv = (char) (cDrive &  0x5f);		// save the upcased drive letter
	pstDrvTab->cDrvNull = 0;						// .. make it ASCIIZ
	memcpy(&pstDrvTab->stBPB,						// copy into the drive table
		   pstBPB,									// .. the boot record BPB
		   sizeof(struct _BPB));					// .. every byte there
	dwSectors = pstBPB->iSectors ?					// q. small sectors filled?
				pstBPB->iSectors :					// a. yes .. get it as count
				pstBPB->dwSecsTotal;				// else .. use larger number
	if (g_bNewWindows)								// q. new windows?
	{												// a. yes .. 
		if (pstBPB->iSecsPerFAT == 0)				// q. fat32?
		{											// a. yes ..
			dwClusters = (dwSectors -				// cal number of clusters
				(pstBPB->iSecsReserved				// ..on the disk
				+ (pstBPB->cFATs					// ..by taking total sectors minus	
						* pstBPB->dwSecsPerFat)))	// ..reserved and fat sectors
				/ pstBPB->cSecsPerCluster;			// ..divided by sector per cluster
			pstDrvTab->dwClusters = dwClusters;		// save number of clusters
			free(pstBoot);							// free the work area 
			return FAT32;							// return it
		}
	}
	dwClusters = (dwSectors -						// cal number of clusters
				(pstBPB->iSecsReserved				// ..on the disk
				+ (pstBPB->cFATs					// ..by taking total sectors minus	
						* pstBPB->iSecsPerFAT)		// ..the sectors that are 
				+ ((pstBPB->iRootEntries * 32)		// ..reserved, those in fats and
					/ pstBPB->iBytesPerSector)))	// ..those in the root directory
				/ pstBPB->cSecsPerCluster;			// ..divided by sector per cluster
	pstDrvTab->dwClusters = dwClusters;				// save number of clusters
	free(pstBoot);									// free the work area
	return (dwClusters > FAT12CLUSTERS) ?			// return FAT16 or UNKNOWN
					(WORD) FAT16 : (WORD) UNKNOWN;	// .. if it's a FAT12
}


// *****************************************************************************
// Determine if a drive is elegible for this program
// Returns TRUE if drive is elegible, else FALSE
// *****************************************************************************

BOOL	DrvEligible(char *szDrv,				// drive to check
					PDRVTABENT pstDrvTab)		// pointer to drive table entry
{
	BOOL		bRc;								// return code work area
	DWORD		dwMaxComponentLen,					// max file name length	
				dwFsFlags;							// file system flags
	char		szFsType[FSNAMESZ],					// file system name buffer
				szVal[10],							// work area for response
				szDrvnm[10];						// work for drive name
	HANDLE		g_hVwin32;							// handle for VWIN32

	if ((GetDriveType(szDrv) != DRIVE_FIXED))		// q. local fixed drive?
		return FALSE;								// a. no .. skip this drive
	bRc = GetVolumeInformation(szDrv,				// get information about the volume	 
							   NULL,				// .. no volume name
							   NULL,				// .. nor name length
							   NULL,				// .. no serial number
							   &dwMaxComponentLen,	// .. get file name length
							   &dwFsFlags,			// .. file systems flags
							   szFsType,			// .. type of file system
							   FSNAMESZ);			// .. and buffer size for type
	if (bRc == FALSE)								// q. did we get the info
		return FALSE;								// a. no .. skip this drive
	if (strncmp(szFsType, FATNAME, strlen(FATNAME)))// q. is it some type of fat?
		return FALSE;								// a. no .. skip the volume
	g_hVwin32 = CreateFile(DEFVWIN32,				// build handler to VWIN32.VXD
						 NULL,						// .. no access code
						 NULL,						// .. no share mode
						 NULL,						// .. no security attrs
						 NULL,						// .. no create directive
						 FILE_FLAG_DELETE_ON_CLOSE,	// .. kill it when we're done
						 NULL);						// .. not based on previous handle
	pstDrvTab->wFatType =							// get/save FAT type
			DetermineFatType(g_hVwin32,				// .. by calling DetermineFatType
							*szDrv,					// .. with VWINT32's handle,
							 pstDrvTab);			// .. the drive id and entry
	bRc =  pstDrvTab->wFatType != UNKNOWN;			// set rc based on type of fat
	CloseHandle(g_hVwin32);							// .. we're done with VWIN32
	sprintf(szDrvnm, "DRIVE%c", *szDrv & 0x5f);		// build the .ini line name
	GetPrivateProfileString("INIT",					// read [INIT] ..
							szDrvnm,				// .. DRIVEx = 
							"N",					// .. default to "N"
							szVal,					// .. put response here
							10,						// .. max chars
							g_strIniFile);			// .. from our ini file
	pstDrvTab->bCheckDrive =						// set the check drive value
			((*szVal & 0x5f) == 'Y');				// .. based on ini file
	if (pstDrvTab->bCheckDrive)						// If drive is selected,
		++g_wNDrivesSelected;						// update selected drive count.
	strcpy((char *)pstDrvTab->acVolName, "(None)");	// start with no name
	g_intFormula = GetPrivateProfileInt("INIT", "FORMULA", 0, g_strIniFile);
	return bRc;										// return final status

}
// *****************************************************************************
// Build the system tray interface
// *****************************************************************************

BOOL BuildSystray(HWND hwnd,						// window to rcv callback messages 					
				  UINT uID,							// id of Icon
				  HICON hicon,						// handle of icon
				  LPSTR lpszTip)					// tip message
{ 
	BOOL bRc;											// return code

	s_ni.hWnd = hwnd;										// .. window handle
	s_ni.uID = uID;											// .. systray icon id
	s_ni.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;			// .. all are valid
	s_ni.uCallbackMessage = MYWM_NOTIFYICON;				// .. call us back with this value
	s_ni.hIcon = hicon;										// .. show this icon
	if (lpszTip)											// q. message valid?
		lstrcpyn(s_ni.szTip, lpszTip, sizeof(s_ni.szTip));	// a. yes .. move it in	
	else													// else ..
		s_ni.szTip[0] = '\0';								// .. kill the message
	bRc = Shell_NotifyIcon(NIM_ADD, &s_ni);					// to the systray..
	return bRc;												// .. and tell caller if success
} 

// *****************************************************************************
// Kill the system tray interface
// *****************************************************************************

BOOL KillSystray(HWND hwnd,							// window to rcv callback messages 					
				 UINT uID)							// id of Icon
{ 
	BOOL bRc;										// return code

	s_ni.hWnd = hwnd;								// .. window handle
	s_ni.uID = uID;									// .. systray icon id
	s_ni.uFlags = NIF_TIP;							// .. tip is valid
	bRc = Shell_NotifyIcon(NIM_DELETE, &s_ni);		// Delete us from systray
	return bRc;										// .. and tell caller if success
} 


// *****************************************************************************
// Put new tip in the systray
// *****************************************************************************
void	ChangeTip(HWND hWnd,						// window used here
				  LPSTR lpszTip)					// change the tip
{
	CString	strZ;										// work string

	strZ = lpszTip;										// copy in the sting
	strZ.TrimRight();									// .. lop off trailing blanks
	lstrcpyn(s_ni.szTip, strZ, sizeof(s_ni.szTip));		// get new message
	s_ni.hWnd = hWnd;									// set the app window
	s_ni.uID = IDR_ICONSMALL0;							// icon identifier
	s_ni.uFlags = NIF_TIP;								// only modify the tip
	Shell_NotifyIcon(NIM_MODIFY, &s_ni);				// .. update that tip
	return;												// .. back to caller
}

// Called during a scan to animate the tray icon.
// Input: iIndex = 0 to 3 = set that specific icon.
//        iIndex < 0 or iIndex > 3 = use next icon in sequence.
void RotateIcon(int iIndex)
{
	static int iIcon = 0;					// Start with first icon.

	if (iIndex >= 0 && iIndex <= 3)			// If requested, set specific icon.
		iIcon = iIndex;
	s_ni.hWnd = g_hMasterWindow;
	s_ni.uID = IDR_ICONSMALL0;
	s_ni.uFlags = NIF_ICON;
	s_ni.hIcon = g_hIcons[iIcon];
	Shell_NotifyIcon(NIM_MODIFY, &s_ni);	// Update the icon.
	iIcon = (iIcon + 1) % 4;				// Next icon in sequence.
} 
// *****************************************************************************
// get the next cluster value
// *****************************************************************************
DWORD	NextClusVal(DWORD dwCluster)			// Get the next cluster
{
	if (g_pZ->wFatType == FAT16)						// q. FAT16?
		return (DWORD)*((WORD *)s_lpacFAT+dwCluster);	// a. yes .. return 16 bit next cluster
	else												// otherwise ..
		return *((DWORD *) s_lpacFAT + dwCluster);		// .. return 32 bit next cluster
}

// *****************************************************************************
// get the next cluster for this chain
// 
// Returns:
//	Cluster Number if OK, 0 if bad or last cluster
// *****************************************************************************
DWORD	NextCluster(DWORD dwCluster)			// Get the next cluster
{
	if (g_pZ->wFatType == FAT16)						// q. is this a FAT16 volume?
	{													// a. yes .. 
		dwCluster = *((WORD *) s_lpacFAT + dwCluster);	// ..get next cluster

		if ((dwCluster < 2)||(dwCluster >= 0xfff0))		// q. bad cluster number/eof?
			dwCluster = 0;								// a. yes .. reset it
	}
	else												// else .. FAT32
	{
		dwCluster = *((DWORD *)s_lpacFAT + dwCluster);	// ..get next cluster number

		if ((dwCluster < 2)							// q. bad cluster number
			|| (dwCluster >= 0x0ffffff0))			// .. or EOF?
			dwCluster = 0;							// a. yes .. reset it
	}
	return dwCluster;								// .. and return the value
}

// *****************************************************************************
// count the free spaces
// *****************************************************************************

void	CountFreeSpaces(void)
{
	DWORD	dwN,									// cluster number to check
			dwX,									// where cluster points
			dwCurTotal = 0,
			dwLastCluster = g_pZ->dwClusters + 2;
	BOOL	bInFS = FALSE;							// TRUE when in a free space
	BOOL    bLowBound = FALSE, bHighBound = FALSE;

	dwN = 2;
	dwX = NextClusVal(dwN++);
	if (dwX)											// If first cluster occupied,
	{
		bLowBound = TRUE;								// set lower bound flag
		while ((dwX != 0) && (dwN <= dwLastCluster))	// find first free cluster.
			dwX = NextClusVal(dwN++);
	}
	if (dwN > dwLastCluster)							// Reached end of disk?
		return;											// No free clusters!
	do		// Loop begins at first free cluster.
	{
		bInFS = TRUE;									// Set "in free space" flag.
		++g_pZ->dwFreeChunks;							// increment total free chunks.
		dwCurTotal = 0;									// Reset total clusters for current free chunk.
		while (dwX == 0 && dwN <= dwLastCluster)		// While current cluster is free,
		{
			++dwCurTotal;								// Increment total.
			dwX = NextClusVal(dwN++);					// Next FAT entry.
		}
		if (dwN > dwLastCluster)						// Reached end of disk yet?
			break;										// Done if so.
		if (bLowBound)									// If lower bound exists, 
		{
			++g_pZ->dwBoundedFreeChunks;				// increment total bounded chunks.
			g_pZ->dwBFreeClusters += dwCurTotal;		// update total bounded clusters.
		}
		else											// otherwise,
		{
			g_pZ->dwUFreeClusters += dwCurTotal;		// update unbounded free clusters.
			bLowBound = TRUE;							// Set lower boundary flag. (From here on, all free
														// blocks must be bounded except for the one at the
														// very end of the disk.
		}
		bInFS = FALSE;									
		while (dwX != 0 && dwN <= dwLastCluster)		// Find start of next free block.
			dwX = NextClusVal(dwN++);
	} 
	while (dwN <= dwLastCluster);						// Keep going till we reach the end of the disk.
	if (bInFS)											// If in free space when we reached end of disk,
	{
		if (g_pZ->dwBoundedFreeChunks > 0)				// and there were any bounded chunks,
			--g_pZ->dwBoundedFreeChunks;				// decrement total bounded chunks (free chunk at
														// end of disk is not bounded).
		g_pZ->dwUFreeClusters += dwCurTotal;			// update total free clusters.
	}
}

// *****************************************************************************
// analyze the requested chain
// *****************************************************************************
void	AnalyzeChain(DWORD dwCluster, BOOL bSystemFile)			// analyze a chain
{
	DWORD		dwNext,			// next cluster number
				dwCount;		// clusters checked
	BOOL		bOK = TRUE;		// assume not fragged

	dwCount = g_pZ->dwClusters;						// set maximum clusters available
	if (dwCluster == 0)								// q. any clusters?
	{												// a. no .. 
		g_pZ->dwZero++;								// .. increment zero-byte files.
		if (bSystemFile)							// System file?
			g_pZ->dwSystemFiles--;					// .. decrement system file count,
		else										// otherwise,
			g_pZ->dwFiles--;						// .. decrement non-system file count.
		return;										// Done.
	}
	while(dwCount--)								// while there may be clusters to check
	{
		dwNext = NextClusVal(dwCluster);			// get the next cluster
		if (bSystemFile)							// Is this a system file?
			++g_pZ->dwSystemClusters;				// Update system file cluster count.
		else										// Otherwise,
		{
			if (bOK)								// q. current clus fragged?
				g_pZ->dwClusUnFrags++;				// a. no .. inc unfragged
			else									// else ..
				g_pZ->dwClusInFrags++;				// .. increment fragged cnt
		}
		if ((dwNext <= 2) 							// q. below minimum?
			 || (dwNext > (g_pZ->dwClusters + 1)))	// .. or above maximum?
		{											// a. yes ..act like EOF
			if (!bSystemFile)						// If not a system file...
			{
				if (bOK)							// q. fragmented file?
					g_pZ->dwOK++;					// a. no .. inc good count
				else
					g_pZ->dwFrag++;					// else .. inc fragged count
			}
			else
			{
				if (bOK)
					g_pZ->dwSysOK++;
				else
					g_pZ->dwSysFrag++;
			}
			return;									// .. done with this file
		}
		else if (dwNext != (dwCluster + 1))  		// q. is it next cluster?
		{											// a. no ..
			bOK = FALSE;							// .. reset the indicator
			if (bSystemFile)						// If system file,
				++g_pZ->dwExtraSysFragments;		// increment extra fragments for system files.
			else									// otherwise
				++g_pZ->dwExtraFragments;			// increment extra fragments for normal files.
		}
		dwCluster = dwNext;							// set next cluster
	}
}

// *****************************************************************************
// Analyze drive's fragmentation 
// *****************************************************************************
void	DrvAnalyze(PDRVTABENT pstDrv,	// analyze the requested drive
				   DWORD dwDirCluster) 	// directory cluster to analyze
{
	static	WORD	wLevel,				// level being entered
					wEntPerSector;		// directory entries per sector
	static	LPBYTE	pDirSector;			// pointer to directory sector area
	static	DWORD	dwFirstDataSector;	// first data sector
	BOOL			bFirstCall,			// first level call
					bRc;				// return code
	DWORD			dwWCluster,			// work cluster number
					dwDirSector,		// current directy sector
					dwDirLastSector,	// last sector for this dir cluster
					dwW;				// work word.
	WORD			wI;					// work int
	PDIRENT			pD;					// pointer to directory entry
	BOOL			bSystemFile;

	if (g_bStopScanNow)								// q. time to stop?
		return;										// a. yes -- done.
	if (!g_bQuickMode)								// If not in quick mode,
		RotateIcon();								// animate the tray icon.
	if (pstDrv != NULL)								// q. drive provided (First level?)
	{												// a. yes .. 
		g_pZ = pstDrv;								// save the drive entry
		s_pB = &pstDrv->stBPB;						// set shorthand BPB pointer
		bFirstCall = TRUE;							// set as first level entry
		wEntPerSector = s_pB->iBytesPerSector;		// calculate entries per sector
		wEntPerSector >>= 5;						// .. divide by 32
		wLevel = 0;									// start the level value
													// Reset ..
		g_pZ->dwFiles = 0;							// files on drive
		g_pZ->dwDirs = 0;							// directories
		g_pZ->dwZero = 0;							// zero-length files
		g_pZ->dwOK = 0;								// non-fragged normal files
		g_pZ->dwSysOK = 0;							// non-fragged system files.
		g_pZ->dwFrag = 0;							// fragged normal files/dirs
		g_pZ->dwSysFrag = 0;						// fragged system files.
		g_pZ->dwExtraFragments = 0;					// fragments on drive
		g_pZ->dwClusInFrags = 0;					// clusters in fragments
		g_pZ->dwClusUnFrags = 0;					// unfragmented clusters
		g_pZ->dwFreeChunks = 0;						// free clusters
		g_pZ->dwBFreeClusters = 0;					// bounded free clusters.
		g_pZ->dwUFreeClusters = 0;					// unbounded free clusters.
		g_pZ->dwBoundedFreeChunks = 0;				// number of free areas
		g_pZ->dwSystemFiles = 0;					// number of system files.
		g_pZ->dwSystemClusters = 0;					// number of clusters in system files.
		g_pZ->dwExtraSysFragments = 0;				// system fragments on drive.
		GetLocalTime(&g_pZ->stLastScan);			// Retrieve current time.
		CountFreeSpaces();							// count up the free spaces
		pDirSector =								// allocate sector read area
			(LPBYTE) malloc(s_pB->iBytesPerSector);	// .. based on drive sector size
		if (g_pZ->wFatType == FAT16)				// q. 16 bit fat?
		{												// a. yes 
			dwDirSector =								// calc first dir sector in root
				(s_pB->cFATs * s_pB->iSecsPerFAT)		// calculate root first sector
						+ s_pB->iSecsReserved;			// .. skip fat & reserved areas
			dwFirstDataSector = dwDirSector				// calc the first data sector
						+ ((s_pB->iRootEntries * 32)	// .. as first + number of sectors
							/ s_pB->iBytesPerSector);	// .. used by dir entries / secsize
			dwDirLastSector = dwFirstDataSector - 1;	// init last root sector
		}
		else										// else .. FAT32
		{
			dwFirstDataSector=						// calculate first data sector
				(s_pB->cFATs * s_pB->dwSecsPerFat)	// .. multiply fat cnt by secs in fat
						+ s_pB->iSecsReserved;		// .. skip fat & reserved areas
			g_pZ->dwDirs++;							// include root in dirs count
			DrvAnalyze(NULL,						// analyze the root directory 
					   s_pB->dwRootCluster);		// .. passing the dir start cluster
			bSystemFile = FALSE;
			AnalyzeChain(s_pB->dwRootCluster, bSystemFile);		// check if root is fragged
			free(pDirSector);						// free the directory sector 
			return;									// .. and return to caller
		}
	}
	else											// on recursive call..
	{
		s_pB = &g_pZ->stBPB;						// set the BPB pointer
		bFirstCall = FALSE;							// show not first entry
		wLevel++;									// .. and increase the level

		dwDirSector = dwFirstDataSector				// calculate the fist sector
					  + ((dwDirCluster - 2)			// .. from the first data sector 
						* s_pB->cSecsPerCluster);	// .. and cluster number
		dwDirLastSector = -1						// calc last cluster. -1 fencepost
						  + (dwDirSector			// .. plus first sector 
						  + s_pB->cSecsPerCluster);	// .. plus sectors per cluster
	}
	// ****************************
	// check directory entries
	// ****************************
	do	
	{											// read down the diretory
		dwW = dwDirSector;							// start with first directory sector
		do											// run this loop ...
		{
			bRc = ReadSectors(g_hVwin32,			// read a directory sector
							  g_pZ->cDrv,			// .. from the drive in process
							  dwW,					// .. sector number
							  1,					// .. for one sector
							  pDirSector);			// .. into "Buffer, the frag slayer"		
			pD = (PDIRENT) pDirSector;				// get first directory entry
			for (wI = wEntPerSector; wI--; pD++)	// while there are entries
			{
				if (*pD->acName == '\0' || ! bRc)	// q. end of directory? or error?
				{									// a. yes ..
					if (bFirstCall)					// q. first level?
					{								// a. yes ..
						free(pDirSector);			// free the directory sector
					}
					return;							// .. done with this level
				}
				if (*pD->acName == DIRDELETED)		// q. deleted file?
					continue;						// .. skip entry
				if (*pD->acName == '.')				// q. . or .. directory?
					continue;						// .. skip entry
				if (pD->bAttr == DIRVIRTENT)		// q. 'virtual' entry?
					continue;						// .. skip entry
				if (pD->bAttr & DIRATTRVOLUME)			// q. volume label?
				{										// a. yes ..
					strncpy((char *)g_pZ->acVolName,	// copy in ..
							(char *)pD->acName,			// .. the file name
							11);						// .. as the label
					g_pZ->acVolName[11] = 0;			// .. assure it's ASCIIZ
					continue;							// .. skip entry
				}
				if (pD->bAttr & DIRATTRDIR)			// q. directory entry?
				{									// a. yes ...
					bSystemFile = FALSE;			// can't be a system file.
					memcpy(s_pszDirName,			// save the directory name
						   pD->acName,				// .. from the dir entry
						   11);						// .. 8.3 type name
					s_pszDirName[11] = 0;
					if ((wLevel == 0)					// q. root level
					   || ((wLevel == 1)				// .. or FAT32							
					   && (g_pZ->wFatType == FAT32)))	// .. root level?
					{									// a. yes ..
						wsprintf((char *) s_pszStatus,	// put in new status message
						"Analyzing %c:\\%s",			// .. drive in process
						g_pZ->cDrv,						// .. drive letter
						s_pszDirName);					// .. and directory
						if (!g_bQuickMode)					// If not in quick mode,
							ChangeTip(g_hMasterWindow,			// change the tip
							(char *) s_pszStatus);
						if (s_lpScanMsg != NULL && !g_bStopScanNow)		// q. s_lpScanMsg provided?
						{												// a. yes ..
					 		s_lpScanMsg->m_lblScanMsg.					// .. set the message
					  			SetWindowText(							// .. in the edit box
									(const char *) s_pszStatus);
							if (g_pRptWin != NULL)									// Property Sheet active?
								if (::GetForegroundWindow() == g_pRptWin->m_hWnd)	// Is it the foreground window?
									s_lpScanMsg->SetForegroundWindow();				// If so, bring scan message window to foreground.
						}
					}
					if (g_pZ->wFatType == FAT32)		// q. fat32 drive?
						dwWCluster =					// a. yes ..
							MAKELONG(pD->wCluster, 		// .. build a long value ..
									 pD->wClusterH);	// .. from the 16 bit values
					else								// otherwise..
						dwWCluster = pD->wCluster;		// use lower part of cluster num
					DrvAnalyze(NULL,				// .. analyze this directory
							   dwWCluster);			// .. passing the dir start cluster
					wLevel--;						// .. fall back a level
					bRc = ReadSectors(g_hVwin32,	// reread directory sector
							  g_pZ->cDrv,			// .. from the drive in process
							  dwW,					// .. sector number
							  1,					// .. for one sector
							  pDirSector);			// .. into our buffer
					g_pZ->dwDirs++;					// increment dir counter
				}	
				else								// otherwise ..
				{
					// Set indicator if only the hidden and system attributes are set.
					bSystemFile = ((pD->bAttr & (DIRATTRSYS | DIRATTRHIDDEN)) == SYSFILEATTR);
					bSystemFile = bSystemFile 			// .. or
								  || (strncmp(			// .. if 
									  (char *)pD->acExt,// .. some kind 
									  "SWP", 3) == 0);	// .. of swap file
					if (bSystemFile)				// If it's a system file,
						++g_pZ->dwSystemFiles;		// increment system file count.
					else							// Otherwise,
						++g_pZ->dwFiles;			// increment normal file count
				}
				if (g_pZ->wFatType == FAT32)		// q. fat32 drive?
			 		dwWCluster =					// a. yes ..
						MAKELONG(pD->wCluster, 		// .. build a long value ..
								 pD->wClusterH);	// .. from the 16 bit values
				else								// otherwise ..
					dwWCluster = pD->wCluster;		// use lower part of cluster num
				AnalyzeChain(dwWCluster, bSystemFile);			// analyze this chain now
			}
			dwW++;									// process next in cluster..
		}
		while (dwW <= dwDirLastSector);				// .. until all sectors processed
		dwDirCluster = NextCluster(dwDirCluster);	// get next cluster
		if (dwDirCluster)							// q. is there a next?
		{											// a. yes ...
			dwDirSector = dwFirstDataSector			// calculate the fist sector
					  + ((dwDirCluster - 2)			// .. from the first data sector 
						* s_pB->cSecsPerCluster);		// .. and cluster number
			dwDirLastSector = -1					// calc last cluster. -1 fencepost
						  + (dwDirSector			// .. plus first sector 
						  + s_pB->cSecsPerCluster);	// .. plus sectors per cluster
		}
	}
	while (dwDirCluster);						// until all cluster processed
}

// *****************************************************************************
// Check fragmentation on a drive
// *****************************************************************************
void	CheckFrag(PDRVTABENT pstDrv)			// check the requested drive
{
	PBPB	pBPB;									// BPB pointer
	BOOL	bRc;   									// boolean return code
	DWORD	dwI;									// work number
	WORD	wSecNo;									// next sector number
	LPBYTE	pW;										// work pointer
	char	szDrv[4];								// drive root name

	strcpy(szDrv, &pstDrv->cDrv);					// copy in drive letter
	strcat(szDrv, ":\\");							// add colon-backslash
	pBPB = &pstDrv->stBPB;							// save addr of BPB
	if (pstDrv->wFatType == FAT16)					// q. 16 bit fat?
		s_lpacFAT = (UCHAR *)						// a. yes ..
					malloc(pBPB->iSecsPerFAT		// .. get space for fat
					* pBPB->iBytesPerSector);		// .. fat sectors x sector size
	else
		s_lpacFAT = (UCHAR *)
					malloc(pBPB->dwSecsPerFat		// .. get space for fat
					* pBPB->iBytesPerSector);		// .. fat sectors x sector size
	if (s_lpacFAT == NULL)							// q. allocate ok?
	{												// a. no ..
		ErrorMsg(ERMNOMEM);							// .. tell the user
		ExitProcess(ERRGENL);						// .. and exit with general error
	}
	g_hVwin32 = CreateFile(DEFVWIN32,				// build handler to VWIN32.VXD
						 NULL,						// .. no access code
						 NULL,						// .. no share mode
						 NULL,						// .. no security attrs
						 NULL,						// .. no create directive
						 FILE_FLAG_DELETE_ON_CLOSE,	// .. kill it when we're done
						 NULL);						// .. not based on previous handle
	pW = s_lpacFAT;									// get initial fat address
	dwI = pstDrv->wFatType == FAT16					// get size of fat in ..
		  ? (DWORD) pBPB->iSecsPerFAT 				// .. 16 bit number of sectors 
		  : pBPB->dwSecsPerFat;						// .. or 32 bit number of sectors
	wSecNo = pBPB->iSecsReserved;					// first fat sector to get
	while (dwI--)									// while there are sectors to get
	{
		bRc = ReadSectors(g_hVwin32,				// use VWIN32 to read the fat 
					  pstDrv->cDrv,					// .. from the drive
					  wSecNo++,						// .. next fat sector
					  1,							// .. for one sector
					  pW);							// .. location to read into
		pW += pBPB->iBytesPerSector;				// .. next memory location
	}
	DrvAnalyze(pstDrv, 0);							// analyze the drive
	pstDrv->bRptRdy = TRUE;							// .. show report ready
	g_bAtLeastOne = TRUE;							// signal that at least one drive has been scanned.
	CloseHandle(g_hVwin32);							// close VWIN32
	free(s_lpacFAT);								// release the FAT!
}


// **************************************************************************
// Calculate Fragmentation Percentage
// **************************************************************************
//
// Input: pD = pointer to drive table entry for desired drive.
//        iFormula = which formula to use:
//					0 = entire disk
//					1 = occupied space only
// Return value: Formula result, or 0 if iFormula parameter is invalid.
float DefragsPct(PDRVTABENT pD, int iFormula)	
{
	DWORD dwNumerator, dwDenominator;

	// Get numerator and denominator for the desired formula
	switch (iFormula)
	{
	case 0:
		dwNumerator = pD->dwExtraFragments + pD->dwBoundedFreeChunks;
		dwDenominator = pD->dwClusters - pD->dwSystemClusters - pD->dwUFreeClusters;
		break;
	case 1:
		dwNumerator = pD->dwExtraFragments + pD->dwExtraSysFragments + pD->dwBoundedFreeChunks;
		dwDenominator = pD->dwClusters - pD->dwUFreeClusters;
		break;
	default:
		return 0.0F;
	}
	if (dwDenominator > 0)								// If denominator is non-zero,
		return (float) dwNumerator / dwDenominator;		// Return percentage
	else												// otherwise
		return 0.0f;									// return zero.
}

/////////////////////////////////////////////////////////////////////////////
// CCrackUpApp

BEGIN_MESSAGE_MAP(CCrackUpApp, CWinApp)
	//{{AFX_MSG_MAP(CCrackUpApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// The one and only CCrackUpApp object

CCrackUpApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CCrackUpApp construction

CCrackUpApp::CCrackUpApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

/////////////////////////////////////////////////////////////////////////////
// CCrackUpApp initialization

BOOL CCrackUpApp::InitInstance()
{
	CString cW;							// Work string
	CString strCmdLine(m_lpCmdLine);	// Copy of command line
	HWND hTarget = NULL;				// Work handle

	s_ni.cbSize = sizeof(NOTIFYICONDATA);	// Set size member of NOTIFYICONDATA structure.
	// See if we're already running.
	// Search for our main window (which this instance hasn't created yet)
	hTarget = FindWindowEx(NULL, NULL, "#32770", "CrackUp Main");
	if (hTarget)										// If found,
	{
		::PostMessage(hTarget, WM_COMMAND, ID_OPEN, 0);	// activate the window.
		return FALSE;									// and we're done.
	}
	// Check for "/QUICK" command line option and set flag accordingly.
	strCmdLine.MakeUpper();
	g_bQuickMode = (strCmdLine.Find("/QUICK") >= 0);
	cW = m_pszHelpFilePath;							// ..and the help file name
	g_strIniFile = cW.Left(cW.GetLength() - 3)		// ..build ini file name
						+ "INI";					// ..by changing HLP to INI
	g_dlgMain.Create(IDD_CRACKUP_DIALOG);		// Create dialog
	m_pMainWnd = &g_dlgMain;					// and set it as our main window.
	return TRUE;
}

