// FileDlg.cpp : implementation file
//
// CrackUp version 1.0
// Copyright  1998 Ziff-Davis, Inc.
// First published in PC Magazine, US Edition, November 3, 1998
// Written by Bob Flanders and Gregory A. Wolking

#include "stdafx.h"
#include "crackup.h"
#include "FileDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Default extensions for each file type.
const char * szDefaultExtensions[4] =
{
	"\0",
	"txt",
	"csv",
	"\0"
};

extern CString g_strIniFile;

/////////////////////////////////////////////////////////////////////////////
// CMyFileDlg

IMPLEMENT_DYNAMIC(CMyFileDlg, CFileDialog)

CMyFileDlg::CMyFileDlg(BOOL bOpenFileDialog, LPCTSTR lpszDefExt, LPCTSTR lpszFileName,
		DWORD dwFlags, LPCTSTR lpszFilter, CWnd* pParentWnd) :
		CFileDialog(bOpenFileDialog, lpszDefExt, lpszFileName, dwFlags, lpszFilter, pParentWnd)
{
	DWORD nIndex;
	// Set flags for explorer-type dialog using a resource-based template and show a Help button.
	m_ofn.Flags |= (OFN_EXPLORER | OFN_ENABLETEMPLATE | OFN_SHOWHELP);
	// Supply identifier of our dialog template. The parent file dialog window uses this template
	// to attach our dialog as a child window at the bottom of its client area.
	// The parent window expands itself as needed to make room for our dialog.
	m_ofn.lpTemplateName = MAKEINTRESOURCE(IDD_FILE_DLG_TEMPLATE);
	// Set instance handle.
	m_ofn.hInstance = AfxGetInstanceHandle();
	// Retrieve filter index from .INI file.
	nIndex = (DWORD) GetPrivateProfileInt("Report Options", "FilterIndex", 1, g_strIniFile);
	if (nIndex < 1 || nIndex > 3)	// Index out of range?
		nIndex = 1;					// default to 1.
	m_ofn.nFilterIndex = nIndex;	// Set filter index
	m_ofn.lpstrDefExt = szDefaultExtensions[nIndex];	// Set default extension for that filter.
}


BEGIN_MESSAGE_MAP(CMyFileDlg, CFileDialog)
	//{{AFX_MSG_MAP(CMyFileDlg)
	ON_WM_CONTEXTMENU()
	//}}AFX_MSG_MAP
	// Note these mapping macros are outside the Class Wizard's block
	// because the CW doesn't understand some of them.
	ON_BN_CLICKED(IDC_chkHeaders, OnchkHeaders)
	ON_BN_CLICKED(IDC_chkLaunch, OnchkLaunch)
	ON_BN_CLICKED(IDC_cmdSaveOptions, OncmdSaveOptions)
	ON_NOTIFY_EX(CDN_HELP, 0, OnHelpButton)
	ON_COMMAND(ID_MY_HELP, OnMyHelp)
END_MESSAGE_MAP()

///////////////////////////
// Member function overrides.

BOOL CMyFileDlg::OnInitDialog() 
{
	char szInit[2];		// Scratch buffer.

	// Attach our checkboxes and static frame
	// to their member variables.
	m_chkHeaders.SubclassDlgItem(IDC_chkHeaders, this);
	m_chkLaunch.SubclassDlgItem(IDC_chkLaunch, this);
	m_cmdSaveOptions.SubclassDlgItem(IDC_cmdSaveOptions, this);
	m_fraOptions.SubclassDlgItem(IDC_fraOptions, this);
	// Call the parent class to initialize the dialog.
	CFileDialog::OnInitDialog();
	// Set context id for the help button.
	SetWindowContextHelpId(IDD_FILE_DLG_TEMPLATE);
	// Retrieve checkbox settings from .INI file.
	GetPrivateProfileString("Report Options", "IncludeHeaders", "N", szInit, 2, g_strIniFile);
	m_bHeaders = (szInit[0] == 'Y') ? 1 : 0;
	GetPrivateProfileString("Report Options", "LaunchReport", "Y", szInit, 2, g_strIniFile);
	m_bLaunch = (szInit[0] == 'Y') ? 1 : 0;
	// Set checkbox state accordingly.
	m_chkHeaders.SetCheck(m_bHeaders);
	m_chkHeaders.EnableWindow(m_ofn.nFilterIndex == 2);
	m_chkLaunch.SetCheck(m_bLaunch);
	// Let Windows handle initial focus.
	return TRUE;
}

// Called after window is built and all controls have been positioned
// We use this message to arrange our controls to fill the space created
// for us in the parent window.
void CMyFileDlg::OnInitDone()
{
	CRect	rectParent,					// Client rectangle of parent window.
			rectMargins(0, 0, 7, 7),	// Desired margins.
			rectThis,					// Our window rect
			rectOptions,				// Options frame rect.
			rectLaunch,					// Launch checkbox rect.
			rectHeaders,				// Headers checkbox rect.
			rectSaveOptions;			// Save options button rect.
	CWnd	*pParent = NULL;
	
	if ((pParent = GetParent()) == NULL)	// If there is no parent (which should never happen),
		return;								// we can't continue.

	pParent->GetClientRect(&rectParent);							// Get client rect of parent window.
	MapDialogRect(&rectMargins);									// Convert desired margins from DLUs to pixels.
	// Get our window rectangles
	GetWindowRect(&rectThis);										// Our dialog rect.
	m_fraOptions.GetWindowRect(&rectOptions);						// "Options" frame.
	m_chkLaunch.GetWindowRect(&rectLaunch);							// "Launch" checkbox.
	m_chkHeaders.GetWindowRect(&rectHeaders);						// "Headers" checkbox.
	m_cmdSaveOptions.GetWindowRect(&rectSaveOptions);
	// Convert rects to client coords relative to our parent.
	pParent->ScreenToClient(&rectOptions);
	pParent->ScreenToClient(&rectLaunch);
	pParent->ScreenToClient(&rectHeaders);
	pParent->ScreenToClient(&rectSaveOptions);
	pParent->ScreenToClient(&rectThis);
	rectThis.right = rectParent.right;								// Set our new right edge.
	rectOptions.right = rectParent.right - rectMargins.right;		// Set right edge of frame with desired margin.
	// Move "Save" button to rightmost edge of the frame.
	rectSaveOptions.OffsetRect((rectOptions.right - rectMargins.right) - rectSaveOptions.right, 0);
	// "Launch" checkbox goes to the right of the "Headers" checkbox
	rectLaunch.left = rectHeaders.right + rectMargins.right;
	// "Launch" checkbox must not overlap the "Save" button.
	rectLaunch.right = rectSaveOptions.left - rectMargins.right;
	m_chkLaunch.MoveWindow(&rectLaunch, FALSE);						// Move the checkbox but don't redraw it yet.
	m_fraOptions.MoveWindow(&rectOptions, FALSE);					// Move the frame but don't redraw it yet.
	m_cmdSaveOptions.MoveWindow(&rectSaveOptions, FALSE);			// Move the button but don't redraw it yet.
	MoveWindow(&rectThis, TRUE);									// Move our window and redraw everything.
}

// Called when user's file type selection changes
void CMyFileDlg::OnTypeChange()
{
	// "Headers" checkbox is only enabled for .CSV format.
	m_chkHeaders.EnableWindow(m_ofn.nFilterIndex == 2);
	// Set default extension for current file type.
	m_ofn.lpstrDefExt = szDefaultExtensions[m_ofn.nFilterIndex];
}

/////////////////////////////////
//  Notification message handlers.

// Called when user clicks the help button.
BOOL CMyFileDlg::OnHelpButton(UINT, NMHDR*, LRESULT*)
{
	AfxGetApp()->WinHelp(IDD_FILE_DLG_TEMPLATE + 0x20000);		// launch WinHelp with the appropriate context
	return TRUE;	// No further processing required by the parent window.
}

/////////////////////////////////////////
//	Windows message handlers.

// Called when user right-clicks in our window.
void CMyFileDlg::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	CPoint ptWhere(point),		// Get coords of mouse pointer.
		ptWork(point);			// Working copy of the point.
	CWnd* pWhich;				// Pointer to target control.
	CRect rectWork;				// Work rectangle.

	// Note: If menu was activated by either the Menu key or Shift + F10,
	// then both x and y coordinates of the point passed will be -1.
	// (A mouse click can never occur at coordinates that are off the screen!)
	if (ptWhere.x < 0 && ptWhere.y < 0)							// Menu requested via keystroke?
	{
		pWhich = pWnd;											// Get pointer to control with the focus.
		pWhich->GetWindowRect(&rectWork);						// Get its rectangle.
		rectWork.NormalizeRect();								// Normalize the rectangle.
		ptWhere.x = rectWork.left + (rectWork.Width() / 2);		// Compute x/y coords to center the
		ptWhere.y = rectWork.top + (rectWork.Height() / 2);		// menu over the control.
	}
	else														// Otherwise,
	{
		// Note: If menu was requested by a mouse click, we can't depend on the pWnd
		// parameter passed to this function -- if the user clicked on a disabled control,
		// pWnd points to the dialog, not the control.
		ScreenToClient(&ptWork);										// Convert to client coords
		pWhich = ChildWindowFromPoint(ptWork, CWP_SKIPTRANSPARENT);		// and see which window user was pointing at.
	}
	if (pWhich != this)		// If target window is not an open part of our client area,
	{
		m_iHelpPopupTopic = pWhich->GetDlgCtrlID() + 0x60000;		// Set pop-up help topic ID.
		CMenu mnu;													// Create a CMenu object.
		mnu.CreatePopupMenu();										// Create a pop-up menu.
		mnu.AppendMenu(MF_STRING, ID_MY_HELP, "&What's This?");		// Add the "What's this?" command.
		mnu.TrackPopupMenu(								// Launch the menu.
			TPM_CENTERALIGN | TPM_LEFTBUTTON,			// Centered, track left mouse button.
			ptWhere.x, ptWhere.y,						// Use calculated coords.
			this,										// If chosen, menu sends command to this window.
			NULL);										// Menu dismissed if user clicks outside it.
	}
	return;
}

///////////////////////////////////////////////
// Control Message handlers

// "Include headers" check box
void CMyFileDlg::OnchkHeaders()
{
	m_bHeaders = m_chkHeaders.GetCheck();	// Copy checkbox state to our member variable.
}

// "Launch report" check box.
void CMyFileDlg::OnchkLaunch()
{
	m_bLaunch = m_chkLaunch.GetCheck();		// Copy checkbox state to member variable.
}

// "What's this?" command from context menu.
void CMyFileDlg::OnMyHelp()
{
	AfxGetApp()->WinHelp(m_iHelpPopupTopic, HELP_CONTEXTPOPUP);
}

// "Save Options" button.
void CMyFileDlg::OncmdSaveOptions()
{
	char buff[8];
	sprintf(buff, "%u", m_ofn.nFilterIndex);
	WritePrivateProfileString("Report Options", "FilterIndex", buff, g_strIniFile);
	buff[1] = '\0';
	buff[0] = m_bHeaders ? 'Y' : 'N';
	WritePrivateProfileString("Report Options",	"IncludeHeaders", buff, g_strIniFile);
	buff[0] = m_bLaunch ? 'Y' : 'N';
	WritePrivateProfileString("Report Options", "LaunchReport", buff, g_strIniFile);
}

